<?php

namespace Drupal\birthday_block\Plugin\Block;

use Drupal\Core\Block\BlockBase;
use Drupal\Core\Config\ImmutableConfig;
use Drupal\Core\Database\Database;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\File\FileUrlGeneratorInterface;
use Drupal\Core\Link;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Url;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides a 'Birthday Block' block.
 *
 * @Block(
 *   id = "birthday_block",
 *   admin_label = @Translation("Birthday Block")
 * )
 */
class BirthdayBlock extends BlockBase implements ContainerFactoryPluginInterface {


  /**
   * The file URL generator.
   *
   * @var \Drupal\Core\File\FileUrlGeneratorInterface
   */
  protected $fileUrlGenerator;
  /**
   * User settings config instance.
   *
   * @var \Drupal\Core\Config\ImmutableConfig
   */
  protected $userSettings;

  /**
   * Constructs a new BlockContentBlock.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin ID for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager service.
   * @param \Drupal\Core\File\FileUrlGeneratorInterface $file_url_generator
   *   The file URL generator.
   * @param \Drupal\Core\Config\ImmutableConfig $user_settings
   *   A user settings config instance.
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, EntityTypeManagerInterface $entity_type_manager, FileUrlGeneratorInterface $file_url_generator, ImmutableConfig $user_settings) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->entityTypeManager = $entity_type_manager;
    $this->fileUrlGenerator = $file_url_generator;
    $this->userSettings = $user_settings;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('entity_type.manager'),
      $container->get('file_url_generator'),
      $container->get('config.factory')->get('user.settings'),
    );
  }

  /**
   * Build Function.
   */
  public function build() {
    $content = $this->getBirthdaysData('block');
    $birthday_content = [];

    if (!empty($content)) {
      $no_of_birthdays_count = 0;
      $no_of_birthdays_to_show = 2;

      if (!empty($content['today'])) {
        $no_of_birthdays_count = count($content['today']);
        if ($no_of_birthdays_count > $no_of_birthdays_to_show) {
          shuffle($content['today']);
          $result = array_slice($content['today'], 0, $no_of_birthdays_to_show);
        }
        elseif ($no_of_birthdays_count == $no_of_birthdays_to_show) {
          shuffle($content['today']);
          $result = $content['today'];
        }
        else {
          $fill_pending_users_data = $no_of_birthdays_to_show - $no_of_birthdays_count;
          shuffle($content['today']);
          if (!empty($content['future'])) {
            $result = $content['today'];
            shuffle($content['future']);
            $fill = array_slice($content['future'], 0, $fill_pending_users_data);
            $result = array_merge($result, $fill);
            $no_of_birthdays_count += count($content['future']);
          }
          else {
            $result = $content['today'];
          }
        }
      }
      elseif (!empty($content['future'])) {
        shuffle($content['future']);
        $result = array_slice($content['future'], 0, $no_of_birthdays_to_show);
        $no_of_birthdays_count = count($content['future']);
      }

      if (!empty($result)) {
        $birthday_items = [];
        foreach ($result as $value) {
          $dob_block_content = [];
          $user = $this->entityTypeManager->getStorage('user')->load($value->uid);
          $user_picture = $user->user_picture->entity ?? NULL;
          $picture_url = $user_picture ? $user_picture->createFileUrl() : $this->fileUrlGenerator->generateAbsoluteString($this->userSettings->get('picture_default') ?? '');
          $user_image_array = [
            'style_name' => 'thumbnail',
            'uri' => $picture_url,
          ];
          $first_name = $user->field_first_name->value ?? '';
          $celebration_date = date('Y-m-d', $value->next_birthday) == date('Y-m-d') ? 'Today' : date('d M', $value->next_birthday);

          $dob_block_content[] = [
            '#markup' => $first_name,
          ];
          $dob_block_content[] = [
            '#theme' => 'image_style',
            '#style_name' => 'thumbnail',
            '#uri' => $user_image_array['uri'],
          ];
          $dob_block_content[] = [
            '#markup' => $celebration_date,
          ];

          $birthday_items[] = [
            '#theme' => 'birthday_block_item',
            '#content' => $dob_block_content,
          ];
        }

        $birthday_content['#theme'] = 'birthday_block';
        $birthday_content['#items'] = $birthday_items;

        if ($no_of_birthdays_count > $no_of_birthdays_to_show) {
          $birthday_content['#items'][] = [
            '#markup' => '<div class="all-bd-link">' . Link::fromTextAndUrl('View All', Url::fromRoute('birthday_block.birthday_page'))->toString() . '</div>',
          ];
        }
      }
    }

    return $birthday_content + [
      '#cache' => ['max-age' => 0],
    ];
  }

  /**
   * Function to get data.
   */
  private function getBirthdaysData($block_content = NULL) {
    $curdate = time();
    $interval_one_week = strtotime('+1 week', $curdate) + 86399;

    $connection = Database::getConnection();
    $query = $connection->select('user__field_dob', 'dob');
    $query->leftJoin('users_field_data', 'u', 'u.uid = dob.entity_id');
    $query->addExpression('UNIX_TIMESTAMP(DATE_ADD(dob.field_dob_value, INTERVAL IF(DAYOFYEAR(dob.field_dob_value) >= DAYOFYEAR(CURDATE() - 1), YEAR(CURDATE()) - YEAR(dob.field_dob_value), YEAR(CURDATE()) - YEAR(dob.field_dob_value) + 1) YEAR))', 'next_birthday');
    $query->fields('dob', ['field_dob_value']);
    $query->fields('u', ['uid']);
    $query->isNotNull('dob.field_dob_value');
    $query->havingCondition('next_birthday', [$curdate, $interval_one_week], 'BETWEEN');
    $query->orderBy('next_birthday', 'ASC');
    $result = $query->execute()->fetchAll();

    $celebration_dob_data = [];
    if (!empty($result)) {
      foreach ($result as $dob_data) {
        if (date('Y-m-d', $dob_data->next_birthday) == date('Y-m-d')) {
          $celebration_dob_data['today'][$dob_data->uid] = $dob_data;
        }
        else {
          $celebration_dob_data['future'][$dob_data->uid] = $dob_data;
        }
      }
    }

    return $celebration_dob_data;
  }

}
